@@ -49,6 +49,9 @@
 
 package com.openexchange.filestore.s3.internal;
 
+import static com.openexchange.filestore.utils.PropertyNameBuilder.optBoolProperty;
+import static com.openexchange.filestore.utils.PropertyNameBuilder.optProperty;
+import static com.openexchange.filestore.utils.PropertyNameBuilder.requireProperty;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
@@ -64,8 +67,6 @@ import java.security.UnrecoverableKeyException;
 import java.security.cert.Certificate;
 import java.security.cert.CertificateException;
 import java.util.Enumeration;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import com.amazonaws.AmazonClientException;
@@ -85,6 +86,7 @@ import com.openexchange.configuration.ConfigurationExceptionCodes;
 import com.openexchange.exception.OXException;
 import com.openexchange.filestore.FileStorage;
 import com.openexchange.filestore.FileStorageProvider;
+import com.openexchange.filestore.utils.PropertyNameBuilder;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 
@@ -121,7 +123,6 @@ public class S3FileStorageFactory implements FileStorageProvider {
      */
     private static final int RANKING = 5634;
 
-    private final ConcurrentMap<URI, S3FileStorage> storages;
     private final ConfigurationService configService;
 
     /**
@@ -132,14 +133,11 @@ public class S3FileStorageFactory implements FileStorageProvider {
     public S3FileStorageFactory(ConfigurationService configService) {
         super();
         this.configService = configService;
-        this.storages = new ConcurrentHashMap<URI, S3FileStorage>();
     }
 
     @Override
     public S3FileStorage getFileStorage(URI uri) throws OXException {
-        try{
-        S3FileStorage storage = storages.get(uri);
-        if (null == storage) {
+        try {
             LOG.debug("Initializing S3 client for {}", uri);
             /*
              * extract filestore ID from authority part of URI
@@ -149,22 +147,17 @@ public class S3FileStorageFactory implements FileStorageProvider {
             /*
              * create client
              */
-            AmazonS3ClientInfo clientInfo = initClient(filestoreID);
+            PropertyNameBuilder nameBuilder = new PropertyNameBuilder("com.openexchange.filestore.s3.");
+            AmazonS3ClientInfo clientInfo = initClient(filestoreID, nameBuilder);
             AmazonS3Client client = clientInfo.client;
-            String bucketName = initBucket(client, filestoreID);
+            String bucketName = initBucket(client, filestoreID, nameBuilder);
             LOG.debug("Using \"{}\" as bucket name.", bucketName);
-            S3FileStorage newStorage = new S3FileStorage(client, clientInfo.encrypted, bucketName, extractFilestorePrefix(uri), clientInfo.chunkSize);
-            storage = storages.putIfAbsent(uri, newStorage);
-            if (null == storage) {
-                storage = newStorage;
-            }
-        }
-            return storage;
-        }catch(OXException ex){
-            if(ex.getCause() instanceof AmazonS3Exception && ((AmazonS3Exception)ex.getCause()).getStatusCode() == 400){
+            return new S3FileStorage(client, clientInfo.encrypted, bucketName, extractFilestorePrefix(uri), clientInfo.chunkSize);
+        } catch (OXException ex) {
+            if (ex.getCause() instanceof AmazonS3Exception && ((AmazonS3Exception) ex.getCause()).getStatusCode() == 400) {
                 // throw a simple OXException here to be able to forward this exception to rmi clients (Bug #42102)
                 throw new OXException(S3ExceptionCode.BadRequest.getNumber(), S3ExceptionMessages.BadRequest_MSG);
-            }else{
+            } else {
                 throw ex;
             }
         }
@@ -192,19 +185,19 @@ public class S3FileStorageFactory implements FileStorageProvider {
      * @return The client
      * @throws OXException
      */
-    private AmazonS3ClientInfo initClient(String filestoreID) throws OXException {
+    private AmazonS3ClientInfo initClient(String filestoreID, PropertyNameBuilder nameBuilder) throws OXException {
         /*
          * prepare credentials
          */
-        String accessKey = requireProperty("com.openexchange.filestore.s3." + filestoreID + ".accessKey");
-        String secretKey = requireProperty("com.openexchange.filestore.s3." + filestoreID + ".secretKey");
+        String accessKey = requireProperty(filestoreID, "accessKey", nameBuilder, configService);
+        String secretKey = requireProperty(filestoreID, "secretKey", nameBuilder, configService);
         BasicAWSCredentials credentials = new BasicAWSCredentials(accessKey, secretKey);
         /*
          * instantiate client
          */
-        ClientConfiguration clientConfiguration = getClientConfiguration(filestoreID);
+        ClientConfiguration clientConfiguration = getClientConfiguration(filestoreID, nameBuilder);
         AmazonS3Client client = null;
-        String encryption = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".encryption", "none");
+        String encryption = optProperty(filestoreID, "encryption", "none", nameBuilder, configService);
         boolean encrypted;
         if (Strings.isEmpty(encryption) || "none".equals(encryption)) {
             /*
@@ -216,28 +209,28 @@ public class S3FileStorageFactory implements FileStorageProvider {
             /*
              * use AmazonS3EncryptionClient
              */
-            client = new AmazonS3EncryptionClient(credentials, getEncryptionMaterials(filestoreID, encryption), clientConfiguration, new CryptoConfiguration());
+            client = new AmazonS3EncryptionClient(credentials, getEncryptionMaterials(filestoreID, encryption, nameBuilder), clientConfiguration, new CryptoConfiguration());
             encrypted = true;
         }
         /*
          * configure client
          */
-        String endpoint = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".endpoint");
+        String endpoint = optProperty(filestoreID, "endpoint", null, nameBuilder, configService);
         if (false == Strings.isEmpty(endpoint)) {
             client.setEndpoint(endpoint);
         } else {
-            String region = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".region", "us-west-2");
+            String region = optProperty(filestoreID, "region", "us-west-2", nameBuilder, configService);
             try {
                 client.setRegion(com.amazonaws.regions.Region.getRegion(Regions.fromName(region)));
             } catch (IllegalArgumentException e) {
                 throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create(e, region);
             }
         }
-        if (configService.getBoolProperty("com.openexchange.filestore.s3." + filestoreID + ".pathStyleAccess", true)) {
+        if (optBoolProperty(filestoreID, "pathStyleAccess", true, nameBuilder, configService)) {
             client.setS3ClientOptions(new S3ClientOptions().withPathStyleAccess(true));
         }
         long chunkSize;
-        String chunkSizeValue = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".chunkSize", "5MB");
+        String chunkSizeValue = optProperty(filestoreID, "chunkSize", "5MB", nameBuilder, configService);
         try {
             chunkSize = ConfigTools.parseBytes(chunkSizeValue);
         } catch (NumberFormatException e) {
@@ -246,22 +239,22 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return new AmazonS3ClientInfo(client, encrypted, chunkSize);
     }
 
-    private ClientConfiguration getClientConfiguration(String filestoreID) {
+    private ClientConfiguration getClientConfiguration(String filestoreID, PropertyNameBuilder nameBuilder) {
         ClientConfiguration clientConfiguration = new ClientConfiguration();
-        String signerOverride = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".signerOverride", "S3SignerType");
+        String signerOverride = optProperty(filestoreID, "signerOverride", "S3SignerType", nameBuilder, configService);
         if (false == Strings.isEmpty(signerOverride)) {
             clientConfiguration.setSignerOverride(signerOverride);
         }
         return clientConfiguration;
     }
 
-    private EncryptionMaterials getEncryptionMaterials(String filestoreID, String encryptionMode) throws OXException {
+    private EncryptionMaterials getEncryptionMaterials(String filestoreID, String encryptionMode, PropertyNameBuilder nameBuilder) throws OXException {
         if (!"rsa".equalsIgnoreCase(encryptionMode)) {
             throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create("Unknown encryption mode: " + encryptionMode);
         }
 
-        String keyStore = requireProperty("com.openexchange.filestore.s3." + filestoreID + ".encryption.rsa.keyStore");
-        String password = requireProperty("com.openexchange.filestore.s3." + filestoreID + ".encryption.rsa.password");
+        String keyStore = requireProperty(filestoreID, "encryption.rsa.keyStore", nameBuilder, configService);
+        String password = requireProperty(filestoreID, "encryption.rsa.password", nameBuilder, configService);
         KeyPair keyPair = extractKeys(keyStore, password);
         return new EncryptionMaterials(keyPair);
     }
@@ -327,11 +320,11 @@ public class S3FileStorageFactory implements FileStorageProvider {
      * @return The bucket name
      * @throws OXException If initialization fails
      */
-    private String initBucket(AmazonS3Client s3client, String filestoreID) throws OXException {
-        String bucketName = requireProperty("com.openexchange.filestore.s3." + filestoreID + ".bucketName");
+    private String initBucket(AmazonS3Client s3client, String filestoreID, PropertyNameBuilder nameBuilder) throws OXException {
+        String bucketName = requireProperty(filestoreID, "bucketName", nameBuilder, configService);
         try {
             if (false == s3client.doesBucketExist(bucketName)) {
-                String region = configService.getProperty("com.openexchange.filestore.s3." + filestoreID + ".region", "us-west-2");
+                String region = optProperty(filestoreID, "region", "us-west-2", nameBuilder, configService);
                 s3client.createBucket(bucketName, Region.fromValue(region));
             }
             return bucketName;
@@ -344,14 +337,6 @@ public class S3FileStorageFactory implements FileStorageProvider {
         }
     }
 
-    private String requireProperty(String propertyName) throws OXException {
-        String property = configService.getProperty(propertyName);
-        if (Strings.isEmpty(property)) {
-            throw ConfigurationExceptionCodes.PROPERTY_MISSING.create(propertyName);
-        }
-        return property;
-    }
-
     /**
      * Extracts the filestore prefix from the configured file store URI, i.e. the 'path' part of the URI.
      *
